#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import jsonpath_rw
import operator
from six import moves


OPERATOR_MAP = {
    '!=': operator.ne,
    '==': operator.eq,
    '=': operator.eq,
    '<=': operator.le,
    '<': operator.lt,
    '>=': operator.ge,
    '>': operator.gt,
}


class Filter(jsonpath_rw.JSONPath):
    """The JSONQuery filter"""

    def __init__(self, expressions):
        self.expressions = expressions

    def find(self, datum):
        if not self.expressions:
            return datum

        datum = jsonpath_rw.DatumInContext.wrap(datum)
        return [jsonpath_rw.DatumInContext(datum.value[i],
                                           path=jsonpath_rw.Index(i),
                                           context=datum)
                for i in moves.range(0, len(datum.value))
                if (len(self.expressions) ==
                    len(list(filter(lambda x: x.find(datum.value[i]),
                                    self.expressions))))]

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.expressions)

    def __str__(self):
        return '[?%s]' % self.expressions


class Expression(jsonpath_rw.JSONPath):
    """The JSONQuery expression"""

    def __init__(self, target, op, value):
        self.target = target
        self.op = op
        self.value = value

    def find(self, datum):
        datum = self.target.find(jsonpath_rw.DatumInContext.wrap(datum))

        if not datum:
            return []
        if self.op is None:
            return datum

        found = []
        for data in datum:
            value = data.value
            if isinstance(self.value, int):
                try:
                    value = int(value)
                except ValueError:
                    continue

            if OPERATOR_MAP[self.op](value, self.value):
                found.append(data)

        return found

    def __eq__(self, other):
        return (isinstance(other, Filter) and
                self.target == other.target and
                self.op == other.op and
                self.value == other.value)

    def __repr__(self):
        if self.op is None:
            return '%s(%r)' % (self.__class__.__name__, self.target)
        else:
            return '%s(%r %s %r)' % (self.__class__.__name__,
                                     self.target, self.op, self.value)

    def __str__(self):
        if self.op is None:
            return '%s' % self.target
        else:
            return '%s %s %s' % (self.target, self.op, self.value)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import operator

import jsonpath_rw

OPERATOR_MAP = {
    '+': operator.add,
    '-': operator.sub,
    '*': operator.mul,
    '/': operator.truediv,
}


class Operation(jsonpath_rw.JSONPath):
    def __init__(self, left, op, right):
        self.left = left
        self.op = OPERATOR_MAP[op]
        self.right = right

    def find(self, datum):
        result = []
        if (isinstance(self.left, jsonpath_rw.JSONPath)
                and isinstance(self.right, jsonpath_rw.JSONPath)):
            left = self.left.find(datum)
            right = self.right.find(datum)
            if left and right and len(left) == len(right):
                for l, r in zip(left, right):
                    try:
                        result.append(self.op(l.value, r.value))
                    except TypeError:
                        return []
            else:
                return []
        elif isinstance(self.left, jsonpath_rw.JSONPath):
            left = self.left.find(datum)
            for l in left:
                try:
                    result.append(self.op(l.value, self.right))
                except TypeError:
                    return []
        elif isinstance(self.right, jsonpath_rw.JSONPath):
            right = self.right.find(datum)
            for r in right:
                try:
                    result.append(self.op(self.left, r.value))
                except TypeError:
                    return []
        else:
            try:
                result.append(self.op(self.left, self.right))
            except TypeError:
                return []
        return [jsonpath_rw.DatumInContext.wrap(r) for r in result]

    def __repr__(self):
        return '%s(%r%s%r)' % (self.__class__.__name__, self.left, self.op,
                               self.right)

    def __str__(self):
        return '%s%s%s' % (self.left, self.op, self.right)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

import jsonpath_rw


SUB = re.compile("sub\(/(.*)/,\s+(.*)\)")
SPLIT = re.compile("split\((.),\s+(\d+),\s+(\d+|-1)\)")


class DefintionInvalid(Exception):
    pass


class Sub(jsonpath_rw.This):
    """Regex subtituor

    Concrete syntax is '`sub(/regex/, repl)`'
    """

    def __init__(self, method=None):
        m = SUB.match(method)
        if m is None:
            raise DefintionInvalid("%s is not valid" % method)
        self.expr = m.group(1).strip()
        self.repl = m.group(2).strip()
        self.regex = re.compile(self.expr)
        self.method = method
        print("%r" % self)

    def find(self, datum):
        datum = jsonpath_rw.DatumInContext.wrap(datum)
        value = self.regex.sub(self.repl, datum.value)
        if value == datum.value:
            return []
        else:
            return [jsonpath_rw.DatumInContext.wrap(value)]

    def __eq__(self, other):
        return (isinstance(other, Sub) and self.method == other.method)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.method)

    def __str__(self):
        return '`sub(/%s/, %s)`' % (self.expr, self.repl)


class Split(jsonpath_rw.This):
    """String splitter

    Concrete syntax is '`split(char, segment, max_split)`'
    """

    def __init__(self, method=None):
        m = SPLIT.match(method)
        if m is None:
            raise DefintionInvalid("%s is not valid" % method)
        self.char = m.group(1)
        self.segment = int(m.group(2))
        self.max_split = int(m.group(3))
        self.method = method

    def find(self, datum):
        datum = jsonpath_rw.DatumInContext.wrap(datum)
        try:
            value = datum.value.split(self.char, self.max_split)[self.segment]
        except Exception:
            return []
        return [jsonpath_rw.DatumInContext.wrap(value)]

    def __eq__(self, other):
        return (isinstance(other, Sub) and self.method == other.method)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.method)

    def __str__(self):
        return '`%s`' % self.method
